@@ -22,6 +22,7 @@ from hypercorn.asyncio import serve
 from grpc import StatusCode
 from urllib.parse import urlparse
 from time import perf_counter
+from starlette_prometheus import metrics, PrometheusMiddleware
 
 supported_methods = ['get', 'put', 'post', 'delete', 'patch']
 
@@ -102,7 +103,18 @@ class GatewayServer(threading.Thread):
 
   def run(self) -> None:
     logging.info(f'starting gateway on port {self.port}')
-    routes = []
+    debug = 'DELPHAI_ENVIRONMENT' in os.environ and os.environ['DELPHAI_ENVIRONMENT'] == 'development'
+    app = Starlette(debug=debug, exception_handlers={HTTPException: self.http_exception})
+    app.add_middleware(AccessLogMiddleware)
+    app.add_middleware(PrometheusMiddleware)
+    app.add_route("/metrics/", metrics)
+    config = Config()
+    config.bind = [f'0.0.0.0:{self.port}']
+    config.loglevel = 'INFO'
+    config.accesslog = '-'
+    config.errorlog = '-'
+    logging.getLogger('hypercorn.access').setLevel(logging.INFO)
+
     for service_handler in self.server._state.generic_handlers:
       if service_handler._name.startswith('grpc.'):
         logging.info(f'skipping service {service_handler._name}')
@@ -121,21 +133,9 @@ class GatewayServer(threading.Thread):
             for supported_method in supported_methods:
               http_path = getattr(http_rule, supported_method)
               if http_path != '':
-                route = Route(http_path, endpoint=self.request_handler, methods=[supported_method])
-                routes.append(route)
+                app.add_route(http_path, route=self.request_handler, methods=[supported_method])
                 self.handlers[http_path] = {'service': service_handler._name, 'method': method_name}
-    middleware = [Middleware(AccessLogMiddleware)]
-    debug = 'DELPHAI_ENVIRONMENT' in os.environ and os.environ['DELPHAI_ENVIRONMENT'] == 'development'
-    app = Starlette(debug=debug,
-                    routes=routes,
-                    exception_handlers={HTTPException: self.http_exception},
-                    middleware=middleware)
-    config = Config()
-    config.bind = [f'0.0.0.0:{self.port}']
-    config.loglevel = 'INFO'
-    config.accesslog = '-'
-    config.errorlog = '-'
-    logging.getLogger('hypercorn.access').setLevel(logging.INFO)
+
     asyncio.run(serve(app, config))
 
 
